#!/bin/bash
#===============================================================================
# Copyright 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#===============================================================================

component=$1

function add_repo {
    wget https://apt.repos.intel.com/intel-gpg-keys/GPG-PUB-KEY-INTEL-SW-PRODUCTS-2023.PUB
    sudo apt-key add GPG-PUB-KEY-INTEL-SW-PRODUCTS-2023.PUB
    rm GPG-PUB-KEY-INTEL-SW-PRODUCTS-2023.PUB
    echo "deb https://apt.repos.intel.com/oneapi all main" | sudo tee /etc/apt/sources.list.d/oneAPI.list
    sudo add-apt-repository -y "deb https://apt.repos.intel.com/oneapi all main"
    sudo apt-get update
}

function install_dpcpp {
    sudo apt-get install                    \
        intel-oneapi-common-vars            \
        intel-oneapi-common-licensing       \
        intel-oneapi-tbb-devel              \
        intel-oneapi-dpcpp-cpp-compiler     \
        intel-oneapi-dev-utilities          \
        intel-oneapi-libdpstd-devel         \
        cmake
    sudo bash -c 'echo libintelocl.so > /etc/OpenCL/vendors/intel-cpu.icd'
    sudo mv -f /opt/intel/oneapi/compiler/latest/linux/lib/oclfpga /opt/intel/oneapi/compiler/latest/linux/lib/oclfpga_
}

function install_mkl {
    sudo apt-get install intel-oneapi-mkl-devel
}

if [ "${component}" == "dpcpp" ]; then
    add_repo
    install_dpcpp
elif [ "${component}" == "mkl" ]; then
    add_repo
    install_mkl
else
    echo "Usage:"
    echo "   $0 [dpcpp|mkl]"
    exit 1
fi
