/* file: Step8LocalNumericTableInputId.java */
/*******************************************************************************
* Copyright 2014-2021 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

/**
 * @ingroup dbscan_compute
 * @{
 */
package com.intel.daal.algorithms.dbscan;

/**
 * <a name="DAAL-CLASS-ALGORITHMS__DBSCAN__STEP8LOCALNUMERICTABLEINPUTID"></a>
 * @brief Available identifiers of input numeric table objects for the DBSCAN algorithm in the eighth step
 *        of the distributed processing mode
 */
public final class Step8LocalNumericTableInputId {
    private int _value;

    /**
     * Constructs the input object identifier using the provided value
     * @param value     Value corresponding to the input object identifier
     */
    public Step8LocalNumericTableInputId(int value) {
        _value = value;
    }

    /**
     * Returns the value corresponding to the input object identifier
     * @return Value corresponding to the input object identifier
     */
    public int getValue() {
        return _value;
    }

    private static final int step8InputClusterStructureValue = 0;
    private static final int step8InputNClustersValue        = 1;

    public static final Step8LocalNumericTableInputId step8InputClusterStructure = new Step8LocalNumericTableInputId(step8InputClusterStructureValue);
       /*!< Input table containing information about current clustering state of observations */
    public static final Step8LocalNumericTableInputId step8InputNClusters = new Step8LocalNumericTableInputId(step8InputNClustersValue);
       /*!< Input table containing the current number of clusters */
}
/** @} */
