/* file: TrainingMethod.java */
/*******************************************************************************
* Copyright 2014-2021 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

/**
 * @defgroup decision_forest_regression_training Training
 * @brief Contains classes for decision forest regression models training
 * @ingroup decision_forest_regression
 * @{
 */
package com.intel.daal.algorithms.decision_forest.regression.training;

/**
 * <a name="DAAL-CLASS-ALGORITHMS__DECISION_FOREST__REGRESSION__TRAINING__TRAININGMETHOD"></a>
 * @brief Available methods for training decision forest regression models
 */
public final class TrainingMethod {

    private int _value;

    /**
     * Constructs the training method object using the provided value
     * @param value     Value corresponding to the training method object
     */
    public TrainingMethod(int value) {
        _value = value;
    }

    /**
     * Returns the value corresponding to the training method object
     * @return Value corresponding to the training method object
     */
    public int getValue() {
        return _value;
    }

    private static final int defaultDenseId = 0;
    private static final int histId = 1;

    /** Default method. Bagging, random choice of features, variance-based impurity. */
    public static final TrainingMethod defaultDense = new TrainingMethod(defaultDenseId);
    /** Hist method. Bagging, random choice of features, variance-based impurity. */
    public static final TrainingMethod hist = new TrainingMethod(histId);
}
/** @} */
